<?php
/**
 * It will add all the settings needed for the plugin.
 *
 * @author  Tyche Softwares
 *
 * @since 2.3.8
 *
 * @package woocommerce-ac
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Wcap_Add_Settings' ) ) {
	/**
	 * It will add all the settings needed for the plugin.
	 *
	 * @since 5.0
	 */
	class Wcap_Add_Settings {

		/**
		 * Update the Schedule Action frequency when the same is updated in the settings.
		 *
		 * @since 8.6
		 */
		public static function wcap_update_cron_interval( $old_value, $new_value ) {
			
			// Now if there's an action scheduled, it needs to be updated with the new frequency
			if ( false !== as_next_scheduled_action( 'woocommerce_ac_send_email_action' ) ) {
				as_unschedule_action( 'woocommerce_ac_send_email_action' );
				
				$new_value = $new_value > 0 ? intval( $new_value ) : 0;
				if ( is_integer( $new_value ) && 0 < $new_value ) {
					$new_interval = $new_value * 60;
					as_schedule_recurring_action( time(), $new_interval, 'woocommerce_ac_send_email_action' );
				}
			}
		}

		/**
		 * Add/Remove the scheduled action based on the setting.
		 *
		 * @param $old_value - Old Value of the setting.
		 * @param $new_value - New Value of the setting.
		 *
		 * @since 8.7.0
		 */
		public static function wcap_use_auto_cron( $old_value, $new_value ) {

			// Now if there's an action scheduled, it needs to be updated with the new frequency
			if ( false !== as_next_scheduled_action( 'woocommerce_ac_send_email_action' ) && '' == $new_value ) {
				as_unschedule_action( 'woocommerce_ac_send_email_action' );
			} else if ( false === as_next_scheduled_action('woocommerce_ac_send_email_action') && 'on' == $new_value ) {
				$cron_interval = intval( get_option( 'wcap_cron_time_duration', 15 ) ) * 60;
				as_schedule_recurring_action( time(), $cron_interval, 'woocommerce_ac_send_email_action' );
			}
		}

		/**
		 * Mark old carts complete.
		 *
		 * @param $old_value - Old Value of the setting.
		 * @param $new_value - New Value of the setting.
		 *
		 * @since 8.11.0
		 */
		public static function wcap_update_ac_enable_cart_emails( $old_value, $new_value ) {
			if ( 'on' === $new_value && 'on' !== $old_value ) {
				$old_carts_time = current_time( 'timestamp' ) - 7 * 86400; // (older than 1 week).
				// Mark carts older than 7 days as abandoned.
				global $wpdb;
				$wpdb->query(
					"UPDATE " . $wpdb->prefix . "ac_abandoned_cart_history SET email_reminder_status = 'complete' WHERE abandoned_cart_time < $old_carts_time AND email_reminder_status = ''"
				);
			}
		}

		/**
		 * It will add all the settings needed for the plugin. All settings are added using the WordPress settings api.
		 *
		 * @hook admin_init
		 *
		 * @since 2.3.8
		 */
		public static function wcap_initialize_plugin_options() {
			$license_type = get_option( 'wcap_edd_license_type', 'enterprise' );
			// First, we register a section.
			add_settings_section(
				'ac_general_settings_section',         // ID used to identify this section and with which to register options.
				__( 'Settings', 'woocommerce-ac' ),                  // Title to be displayed on the administration page.
				array( 'Wcap_Add_Settings', 'wcap_general_options_callback' ), // Callback used to render the description of the section.
				'woocommerce_ac_page'     // Page on which to add this section of options.
			);

			add_settings_field(
				'ac_enable_cart_emails',
				__( 'Enable abandoned cart emails', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_enable_cart_emails_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'Yes, enable the abandoned cart emails.', 'woocommerce-ac' ) )
			);
			add_settings_field(
				'ac_cart_abandoned_time',
				__( 'Cart abandoned cut-off time for logged-in users', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_cart_abandoned_time_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'minutes<br>For logged-in users consider cart abandoned after X minutes of item being added to cart & order not placed.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'ac_cart_abandoned_time_guest',
				__( 'Cart abandoned cut-off time for guest users', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_cart_abandoned_time_guest_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'minutes<br>For guest users & visitors consider cart abandoned after X minutes of item being added to cart & order not placed.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'ac_delete_abandoned_order_days',
				__( 'Automatically Delete Abandoned Orders after X days', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_delete_abandoned_orders_days_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'Automatically delete abandoned cart orders after X days.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'ac_email_admin_on_recovery',
				__( 'Email admin On Order Recovery', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_email_admin_on_recovery_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'Sends email to Admin if an Abandoned Cart Order is recovered.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'ac_disable_guest_cart_email',
				__( 'Do not track carts of guest users', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_disable_guest_cart_email_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'Abandoned carts of guest users will not be tracked.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'ac_track_guest_cart_from_cart_page',
				__( 'Start tracking from Cart Page', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_track_guest_cart_from_cart_page_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'Enable tracking of abandoned products & carts even if customer does not visit the checkout page or does not enter any details on the checkout page like Name or Email. Tracking will begin as soon as a visitor adds a product to their cart and visits the cart page.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'ac_disable_logged_in_cart_email',
				__( 'Do not track carts of logged-in users', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_disable_logged_in_cart_email_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'Abandoned carts of logged-in users will not be tracked.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'ac_cart_abandoned_after_x_days_order_placed',
				__( 'Send reminder emails for newly abandoned carts after X days of order placement', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_cart_abandoned_after_x_days_order_placed_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'Reminder emails will be sent for newly abandoned carts only after X days of a previously placed order for a user with the same email address as that of the abandoned cart', 'woocommerce-ac' ) )
			);

			if ( 'starter' !== $license_type ) {
				add_settings_field(
					'ac_capture_email_from_forms',
					__( 'Capture email address from custom fields.', 'woocommerce-ac' ),
					array( 'Wcap_Add_Settings', 'wcap_capture_email_from_forms' ),
					'woocommerce_ac_page',
					'ac_general_settings_section',
					array( __( 'Enable this setting to capture email address from other form fields.', 'woocommerce-ac' ) )
				);

				add_settings_field(
					'ac_email_forms_classes',
					__( 'Class names of the form fields.', 'woocommerce-ac' ),
					array( 'Wcap_Add_Settings', 'wcap_email_forms_classes' ),
					'woocommerce_ac_page',
					'ac_general_settings_section',
					array( __( 'Enter class names of fields separated by commas from where email needs to be captured.', 'woocommerce-ac' ) )
				);
			}

			add_settings_field(
				'ac_capture_email_address_from_url',
				__( 'Capture Email address from URL', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_capture_email_address_from_url' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'If your site URL contain the same key, then it will capture it as an email address of customer.', 'woocommerce-ac' ) )
			);

			add_settings_section(
				'ac_gdpr_settings_section',           // ID used to identify this section and with which to register options.
				__( 'GDPR Consent', 'woocommerce-ac' ),      // Title to be displayed on the administration page.
				array( 'Wcap_Add_Settings', 'wcap_gdpr_callback' ), // Callback used to render the description of the section.
				'woocommerce_ac_page'     // Page on which to add this section of options.
			);

			add_settings_field(
				'wcap_enable_gdpr_consent',
				__( 'Enable GDPR Notice', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_enable_gdpr_consent_callback' ),
				'woocommerce_ac_page',
				'ac_gdpr_settings_section',
				array( __( 'Enable this setting to display a notice informing customers that their email and cart data are saved to send abandonment reminders.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_guest_cart_capture_msg',
				__( 'Message to be displayed for Guest users when tracking their carts', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_guest_cart_capture_msg_callback' ),
				'woocommerce_ac_page',
				'ac_gdpr_settings_section',
				array( __( '<br>In compliance with GDPR, add a message on the Checkout page and Email Address Capture pop-up to inform Guest users of how their data is being used.<br><i>For example: Your email address will help us support your shopping experience throughout the site. Please check our Privacy Policy to see how we use your personal data.</i>', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_logged_cart_capture_msg',
				__( 'Message to be displayed for registered users when tracking their carts.', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_logged_cart_capture_msg_callback' ),
				'woocommerce_ac_page',
				'ac_gdpr_settings_section',
				array( __( '<br>In compliance with GDPR, add a message on the Shop & Product pages to inform Registered users of how their data is being used.<br><i>For example: Please check our Privacy Policy to see how we use your personal data.</i>', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_gdpr_allow_opt_out',
				__( 'Allow the visitor to opt out of cart tracking.', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_gdpr_allow_opt_out_callback' ),
				'woocommerce_ac_page',
				'ac_gdpr_settings_section',
				array( __( '<br>In compliance with GDPR, allow the site visitor (guests & registered users) to opt out from cart tracking. This message will be displayed in conjunction with the GDPR message above.</i>', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_gdpr_opt_out_message',
				__( 'Message to be displayed when the user chooses to opt out of cart tracking.', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_gdpr_opt_out_msg_callback' ),
				'woocommerce_ac_page',
				'ac_gdpr_settings_section',
				array( __( '<br>Message to be displayed when the user chooses to opt out of cart tracking.</i>', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_enable_sms_consent',
				__( 'Enable SMS Consent', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_enable_sms_consent_callback' ),
				'woocommerce_ac_page',
				'ac_gdpr_settings_section',
				array( __( 'Enable this setting to display a notice informing customers that their phone number and cart data are saved to send abandonment reminders.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_sms_consent_msg',
				__( 'SMS Consent Message', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_sms_consent_msg_callback' ),
				'woocommerce_ac_page',
				'ac_gdpr_settings_section',
				array( __( 'Message to be displayed below the Phone field at Checkout informing users that their phone number and cart data are saved to send abandonment reminders.', 'woocommerce-ac' ) )
			);

			add_settings_section(
				'ac_coupon_settings_section',           // ID used to identify this section and with which to register options.
				__( 'Coupon Settings', 'woocommerce-ac' ),      // Title to be displayed on the administration page.
				array( 'Wcap_Add_Settings', 'wcap_coupon_callback' ), // Callback used to render the description of the section.
				'woocommerce_ac_page'     // Page on which to add this section of options.
			);

			add_settings_field(
				'wcap_delete_coupon_data',
				__( 'Delete Coupons Automatically', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_deleting_coupon_data' ),
				'woocommerce_ac_page',
				'ac_coupon_settings_section',
				array( __( 'Enable this setting if you want to completely remove the expired and used coupon code automatically every 15 days.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_delete_coupon_data_manually',
				__( 'Delete Coupons Manually', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_deleting_coupon_data_manually' ),
				'woocommerce_ac_page',
				'ac_coupon_settings_section',
				array( __( 'If you want to completely remove the expired and used coupon code now then click on "Delete" button.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcac_delete_plugin_data',
				__( 'Remove Data on Uninstall?', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_deleting_plugin_data' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'Enable this setting if you want to completely remove Abandoned Cart data when plugin is deleted.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_atc_close_icon_add_product_to_cart',
				__( 'Add Product to Cart when Close Icon is clicked?', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_atc_close_icon_add_product_to_cart_callback' ),
				'woocommerce_ac_page',
				'ac_general_settings_section',
				array( __( 'Enable this setting if you want the product to the added to cart when the user clicks on the Close Icon in the Add to Cart Popup Modal.', 'woocommerce-ac' ) )
			);

			/**
			 * New section for the Adding the abandoned cart setting.
			 *
			 * @since 4.7
			 */
			add_settings_section(
				'ac_email_settings_section',           // ID used to identify this section and with which to register options.
				__( 'Settings for abandoned cart recovery emails', 'woocommerce-ac' ),      // Title to be displayed on the administration page.
				array( 'Wcap_Add_Settings', 'wcap_email_callback' ), // Callback used to render the description of the section.
				'woocommerce_ac_page'     // Page on which to add this section of options.
			);

			add_settings_field(
				'wcap_from_name',
				__( '"From" Name', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_from_name_callback' ),
				'woocommerce_ac_page',
				'ac_email_settings_section',
				array( __( 'Enter the name that should appear in the email sent.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_from_email',
				__( '"From" Address', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_from_email_callback' ),
				'woocommerce_ac_page',
				'ac_email_settings_section',
				array( __( 'Email address from which the reminder emails should be sent.', 'woocommerce-ac' ) . '<strong>' . __( 'Note' ) . ':</strong>' . __( 'This setting shall be applicable only when PHP mail function is used by your Hosting Provider. If SMTP mail plugins are used or if mail configuration is based on SMTP then this setting wont be applicable.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_reply_email',
				__( 'Send Reply Emails to', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_reply_email_callback' ),
				'woocommerce_ac_page',
				'ac_email_settings_section',
				array( __( 'When a contact receives your email and clicks reply, which email address should that reply be sent to?', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_product_image_size',
				__( 'Product Image( H x W )', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_product_image_size_callback' ),
				'woocommerce_ac_page',
				'ac_email_settings_section',
				array( __( 'This setting affects the dimension of the product image in the abandoned cart reminder email.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_product_name_redirect',
				__( 'Product Name Redirects to', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_product_name_redirect_callback' ),
				'woocommerce_ac_page',
				'ac_email_settings_section',
				array( __( 'Select the page where product name in reminder emails should redirect to.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_add_utm_to_links',
				__( 'UTM parameters to be added to all the links in reminder emails', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_add_utm_to_links_callback' ),
				'woocommerce_ac_page',
				'ac_email_settings_section',
				array( __( 'UTM parameters that should be added to all the links in reminder emails.', 'woocommerce-ac' ) )
			);

			/*
			 * New section for the Updating the cron job time
			 * Since @: 4.0
			 */

			add_settings_section(
				'ac_cron_job_settings_section',           // ID used to identify this section and with which to register options.
				__( 'Setting for sending Emails & SMS using Action Scheduler', 'woocommerce-ac' ),      // Title to be displayed on the administration page.
				array( 'Wcap_Add_Settings', 'wcap_cron_job_callback' ), // Callback used to render the description of the section.
				'woocommerce_ac_page'     // Page on which to add this section of options.
			);
			add_settings_field(
				'wcap_use_auto_cron',
				__( 'Send  Abandoned cart reminders automatically using Action Scheduler', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_use_auto_cron_callback' ),
				'woocommerce_ac_page',
				'ac_cron_job_settings_section',
				array(
					sprintf(
						/* translators: %1$s and %2$s is replaced with Moving to Action scheduler link and Support link respectively */
						__( 'Enabling this setting will send the abandoned cart reminders to the customer after the set time. Please visit %1$s for more information on the Action Scheduler.  If disabled, automated abandoned cart reminders will not be sent by the plugin. Carts will be tracked based on the settings, but no reminders will be sent.', 'woocommerce-ac' ),
						'<a href="https://www.tychesoftwares.com/moving-to-the-action-scheduler-library/?utm_source=AcProNotice&utm_medium=link&utm_campaign=AbandonCartPro" target="_blank">here</a>',
						'<a href= mailto:support@tychesoftwares.com>contact us</a>'
					),
				)
			);

			add_settings_field(
				'wcap_cron_time_duration',
				__( 'Run Automated Scheduler after X minutes', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_cron_time_duration_callback' ),
				'woocommerce_ac_page',
				'ac_cron_job_settings_section',
				array( __( 'The duration in minutes after which an action should be automatically scheduled to send email, SMS & FB reminders to customers.', 'woocommerce-ac' ) )
			);

			// Setting section and field for Email reports.
			add_settings_section(
				'ac_email_reports_section',
				__( 'Reports', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'ac_email_reports_section_callback' ),
				'woocommerce_ac_email_reports_page'
			);

			add_settings_field(
				'wcap_email_reports_frequency',
				__( 'Frequency with which reports should be sent', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_email_reports_frequency_callback' ),
				'woocommerce_ac_email_reports_page',
				'ac_email_reports_section',
				array( __( 'Send emails on weekly/monthly basis.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_email_reports_emails_list',
				__( 'Send report emails to', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_email_reports_emails_list_callback' ),
				'woocommerce_ac_email_reports_page',
				'ac_email_reports_section',
				array( __( 'Enter a list of email addresses. Each email address should be on a new line.', 'woocommerce-ac' ) )
			);

			// Setting section and field for license options.
			add_settings_section(
				'ac_general_license_key_section',
				__( 'Plugin License Options', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_general_license_key_section_callback' ),
				'woocommerce_ac_license_page'
			);

			add_settings_field(
				'edd_sample_license_key_ac_woo',
				__( 'License Key', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_edd_sample_license_key_ac_woo_callback' ),
				'woocommerce_ac_license_page',
				'ac_general_license_key_section',
				array( __( 'Enter your license key.', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'activate_license_key_ac_woo',
				__( 'Activate License', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_activate_license_key_ac_woo_callback' ),
				'woocommerce_ac_license_page',
				__( 'ac_general_license_key_section', 'woocommerce-ac' )
			);

			/**
			 * New section for custom restrict settings
			 * @since 4.1
			 */
			add_settings_section(
				'ac_restrict_settings_section',           // ID used to identify this section and with which to register options.
				__( 'Rules to exclude capturing abandoned carts', 'woocommerce-ac' ),      // Title to be displayed on the administration page.
				array( 'Wcap_Add_Settings', 'wcap_custom_restrict_callback' ), // Callback used to render the description of the section.
				'woocommerce_ac_page'     // Page on which to add this section of options.
			);

			add_settings_field(
				'wcap_restrict_ip_address',
				__( 'Do not capture abandoned carts for these IP addresses', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_restrict_ip_address_callback' ),
				'woocommerce_ac_page',
				'ac_restrict_settings_section',
				array(
					sprintf(
						/* translators: %1$s and %2$s is replaced with example Wild card IPs to be blocked */
						__( 'The carts abandoned from these IP addresses will not be tracked by the plugin. Accepts wildcards, e.g %1$s will block all IP addresses which starts from %2$s. ', 'woocommerce-ac' ),
						'<code>192.168.*</code>',
						'"192.168"'
					) . '<i>' . __( 'Separate IP addresses with commas.', 'woocommerce-ac' ) . '</i>',
				)
			);

			add_settings_field(
				'wcap_restrict_email_address',
				__( 'Do not capture abandoned carts for these email addresses', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_restrict_email_address_callback' ),
				'woocommerce_ac_page',
				'ac_restrict_settings_section',
				array( __( 'The carts abandoned using these email addresses will not be tracked by the plugin. ', 'woocommerce-ac' ). '<i>' . __( 'Separate IP addresses with commas.', 'woocommerce-ac' ) . '</i>' )
			);

			add_settings_field(
				'wcap_restrict_domain_address',
				__( 'Do not capture abandoned carts for email addresses from these domains', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_restrict_domain_address_callback' ),
				'woocommerce_ac_page',
				'ac_restrict_settings_section',
				array( __( 'The carts abandoned from email addresses with these domains will not be tracked by the plugin. ', 'woocommerce-ac' ) )
			);

			add_settings_field(
				'wcap_restrict_countries',
				__( 'Do not capture carts from countries', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_restrict_countries_callback' ),
				'woocommerce_ac_page',
				'ac_restrict_settings_section',
				array(
					__( 'The carts abandoned from these countries will not be tracked by the plugin. ', 'woocommerce-ac' ),
					'class' => 'wcap_restrict_countries',
				)
			);

			if ( 'starter' !== $license_type  ) {
				add_settings_section(
					'ac_unsubscribe_section',
					__( 'Unsubscribe Settings', 'woocommerce-ac' ),
					array( 'Wcap_Add_Settings', 'wcap_unsubscribe_options_callback' ),
					'woocommerce_ac_page'
				);

				$doc_link = 'https://www.tychesoftwares.com/docs/docs/abandoned-cart-pro-for-woocommerce/unsubscribe-landing-page-options';
				add_settings_field(
					'wcap_unsubscribe_landing_page',
					__( 'Unsubscribe Landing Page', 'woocommerce-ac' ),
					array( 'Wcap_Add_Settings', 'wcap_unsubscribe_landing_page_callback' ),
					'woocommerce_ac_page',
					'ac_unsubscribe_section',
					array(
						sprintf(
							/* translators: %1$s is replaced with Documentation link about unsubscribing*/
							__( 'Select a source where the user must be redirected when an Unsubscribe link is clicked from reminders sent. For details, please check the %s', 'woocommerce-ac' ),
							"<a href='$doc_link' target='_blank'>" . __( 'documentation') . "</a>." 
						),
					)
				);

				add_settings_field(
					'wcap_unsubscribe_custom_content',
					__( 'Custom Content', 'woocommerce-ac' ),
					array( 'Wcap_Add_Settings', 'wcap_unsubscribe_custom_content_callback' ),
					'woocommerce_ac_page',
					'ac_unsubscribe_section',
					array('')
				);

				add_settings_field(
					'wcap_unsubscribe_custom_wp_page',
					__( 'Select Custom WordPress page', 'woocommerce-ac' ),
					array( 'Wcap_Add_Settings', 'wcap_unsubscribe_custom_wp_page_callback' ),
					'woocommerce_ac_page',
					'ac_unsubscribe_section',
					array(__( 'Select a WordPress page to redirect to.', 'woocommerce-ac' ) )
				);
			}

			/**
			 * New Settings for SMS Notifications
			 */
			add_settings_section(
				'wcap_sms_settings_section',        // ID used to identify this section and with which to register options.
				__( 'Twilio', 'woocommerce-ac' ),     // Title to be displayed on the administration page.
				array( 'Wcap_Add_Settings', 'wcap_sms_settings_section_callback' ),       // Callback used to render the description of the section.
				'woocommerce_ac_sms_page'               // Page on which to add this section of options.
			);

			add_settings_field(
				'wcap_enable_sms_reminders',
				__( 'Enable SMS', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_enable_sms_reminders_callback' ),
				'woocommerce_ac_sms_page',
				'wcap_sms_settings_section',
				array( '<i>' . __( 'Enable the ability to send reminder SMS for abandoned carts.', 'woocommerce-ac' ) . '</i>' )
			);

			add_settings_field(
				'wcap_sms_from_phone',
				__( 'From', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_sms_from_phone_callback' ),
				'woocommerce_ac_sms_page',
				'wcap_sms_settings_section',
				array( '<i>' . __( 'Must be a Twilio phone number (in E.164 format) or alphanumeric sender ID.', 'woocommerce-ac' ) . '</i>'  )
			);

			add_settings_field(
				'wcap_sms_account_sid',
				__( 'Account SID', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_sms_account_sid_callback' ),
				'woocommerce_ac_sms_page',
				'wcap_sms_settings_section',
				array( '' )
			);

			add_settings_field(
				'wcap_sms_auth_token',
				__( 'Auth Token', 'woocommerce-ac' ),
				array( 'Wcap_Add_Settings', 'wcap_sms_auth_token_callback' ),
				'woocommerce_ac_sms_page',
				'wcap_sms_settings_section',
				array( '' )
			);

			// Finally, we register the fields with WordPress
			register_setting(
				'woocommerce_ac_settings',
				'ac_enable_cart_emails'
			);
			register_setting(
				'woocommerce_ac_settings',
				'ac_cart_abandoned_time',
				array( 'Wcap_Add_Settings', 'wcap_cart_time_validation' )
			);
			register_setting(
				'woocommerce_ac_settings',
				'ac_cart_abandoned_time_guest',
				array( 'Wcap_Add_Settings', 'wcap_cart_time_guest_validation' )
			);
			register_setting(
				'woocommerce_ac_settings',
				'ac_delete_abandoned_order_days',
				array( 'Wcap_Add_Settings', 'wcap_delete_days_validation' )
			);
			register_setting(
				'woocommerce_ac_settings',
				'ac_email_admin_on_recovery'
			);
			register_setting(
				'woocommerce_ac_settings',
				'ac_email_admin_on_abandoned'
			);

			register_setting(
				'woocommerce_ac_settings',
				'ac_disable_guest_cart_email'
			);
			register_setting(
				'woocommerce_ac_settings',
				'ac_disable_logged_in_cart_email'
			);

			register_setting(
				'woocommerce_ac_settings',
				'ac_cart_abandoned_after_x_days_order_placed'/*,
				array( 'Wcap_Add_Settings', 'wcap_delete_days_validation' )*/
			);

			if ( 'starter' !== $license_type ) {
				register_setting(
					'woocommerce_ac_settings',
					'ac_capture_email_from_forms'
				);
				register_setting(
					'woocommerce_ac_settings',
					'ac_email_forms_classes'
				);
			}

			register_setting(
				'woocommerce_ac_settings',
				'ac_capture_email_address_from_url'
			);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_enable_gdpr_consent'
			);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_guest_cart_capture_msg'
			);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_logged_cart_capture_msg'
			);

	      	register_setting(
		        'woocommerce_ac_settings',
		        'wcap_gdpr_allow_opt_out'
	      	);
	            
	      	register_setting(
		        'woocommerce_ac_settings',
		        'wcap_gdpr_opt_out_message'
	      	);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_enable_sms_consent'
			);
			register_setting(
				'woocommerce_ac_settings',
				'wcap_sms_consent_msg'
			);

	      	register_setting(
	        	'woocommerce_ac_settings',
	        	'wcap_delete_coupon_data'
	      	);

	      	register_setting(
	        	'woocommerce_ac_settings',
	        	'wcac_delete_plugin_data'
	      	);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_atc_close_icon_add_product_to_cart'
			);

			register_setting(
				'woocommerce_ac_settings',
				'ac_track_guest_cart_from_cart_page'
			);
			register_setting(
				'woocommerce_ac_email_reports_frequency_settings',
				'wcap_email_reports_frequency'
			);
			register_setting(
				'woocommerce_ac_email_reports_frequency_settings',
				'wcap_email_reports_emails_list'
			);
			register_setting(
				'woocommerce_ac_license',
				'edd_sample_license_key_ac_woo'
			);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_cron_time_duration'
			);
			register_setting(
				'woocommerce_ac_settings',
				'wcap_use_auto_cron'
			);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_restrict_ip_address'
			);
			register_setting(
				'woocommerce_ac_settings',
				'wcap_restrict_email_address'
			);
			register_setting(
				'woocommerce_ac_settings',
				'wcap_restrict_domain_address'
			);
			register_setting(
				'woocommerce_ac_settings',
				'wcap_restrict_countries'
			);
			if ( 'starter' !== $license_type  ) {
				register_setting(
					'woocommerce_ac_settings',
					'wcap_unsubscribe_landing_page'
				);
				register_setting(
					'woocommerce_ac_settings',
					'wcap_unsubscribe_custom_content'
				);
				register_setting(
					'woocommerce_ac_settings',
					'wcap_unsubscribe_custom_wp_page'
				);
			}

			register_setting(
				'woocommerce_ac_settings',
				'wcap_from_name'
			);
			register_setting(
				'woocommerce_ac_settings',
				'wcap_from_email'
			);
			register_setting(
				'woocommerce_ac_settings',
				'wcap_reply_email'
			);
			register_setting(
				'woocommerce_ac_settings',
				'wcap_product_image_height'
			);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_product_image_width'
			);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_product_name_redirect'
			);

			register_setting(
				'woocommerce_ac_settings',
				'wcap_add_utm_to_links'
			);

			register_setting(
				'woocommerce_sms_settings',
				'wcap_enable_sms_reminders'
			);

			register_setting(
				'woocommerce_sms_settings',
				'wcap_sms_from_phone'
			);

			register_setting(
				'woocommerce_sms_settings',
				'wcap_sms_account_sid'
			);

			register_setting(
				'woocommerce_sms_settings',
				'wcap_sms_auth_token'
			);

			do_action( 'wcap_add_new_settings' );
		}

		/**
		 * Abandoned cart time field validation for loggedin users.
		 *
		 * @param int | string $input Input of the field Abandoned cart cut off time
		 *
		 * @return int | string $output Error message or the input value
		 *
		 * @since 2.3.8
		 */
		public static function wcap_cart_time_validation( $input ) {
			$output = '';
			if ( $input != '' && ( is_numeric( $input ) && $input > 0 ) ) {
				$output = stripslashes( $input );
			} else {
				add_settings_error( 'ac_cart_abandoned_time', 'error found', __( 'Abandoned cart cut off time for logged-in users should be numeric and has to be greater than 0.', 'woocommerce-ac' ) );
			}

			return $output;
		}

		/**
		 * Abandoned cart time field validation for guest users.
		 *
		 * @param int | string $input input of the field Abandoned cart cut off time
		 *
		 * @return int | string $output Error message or the input value
		 *
		 * @since 2.3.8
		 */
		public static function wcap_cart_time_guest_validation( $input ) {
			$output = '';
			if ( $input != '' && ( is_numeric( $input ) && $input > 0 ) ) {
				$output = stripslashes( $input );
			} else {
				add_settings_error( 'ac_cart_abandoned_time_guest', 'error found', __( 'Abandoned cart cut off time for guest users should be numeric and has to be greater than 0.', 'woocommerce-ac' ) );
			}

			return $output;
		}

		/**
		 * Validation for automatically delete abandoned carts after X days.
		 *
		 * @param int | string $input input of the field Abandoned cart cut off time
		 *
		 * @return int | string $output Error message or the input value
		 *
		 * @since 2.3.8
		 */
		public static function wcap_delete_days_validation( $input ) {
			$output = '';
			if ( is_numeric( $input ) && $input > 0 ) {
				$output = stripslashes( $input );
			} else {
				$output = '365';
				add_settings_error( 'ac_delete_abandoned_order_days', 'error found', __( 'Automatically Delete Abandoned Orders after X days has to be greater than 0.', 'woocommerce-ac' ) );
			}

			return $output;
		}

		/**
		 * Call back for the ac_general_settings_section section.
		 *
		 * @since 2.3.8
		 */
		public static function wcap_general_options_callback() {
		}

		/**
		 * Callback for enable cart emails field.
		 *
		 * @param array $args Argument given while adding the field.
		 *
		 * @since 2.3.8
		 */
		public static function wcap_enable_cart_emails_callback( $args ) {
			// First, we read the option.
			$enable_cart_emails = get_option( 'ac_enable_cart_emails' );
			// This condition added to avoid the notice displayed while Check box is unchecked.
			if ( isset( $enable_cart_emails ) && $enable_cart_emails == '' ) {
				$enable_cart_emails = 'off';
			}
			// Next, we update the name attribute to access this element's ID in the context of the display options array.
			// We also access the show_header element of the options collection in the call to the checked() helper function.
			echo '<input type="checkbox" id="ac_enable_cart_emails" name="ac_enable_cart_emails" value="on" ', checked( 'on', $enable_cart_emails, false ), '/>';
			// Here, we'll take the first argument of the array and add it to a label next to the check box.
			echo '<label for="ac_enable_cart_emails"> ', esc_attr( $args[0] ), '</label>';
		}

		/**
		 * Callback for Abandoned cart time field for Logged in user.
		 *
		 * @param array $args Argument given while adding the field.
		 *
		 * @since 2.3.8
		 */
		public static function wcap_cart_abandoned_time_callback( $args ) {
			// First, we read the option.
			$cart_abandoned_time = get_option( 'ac_cart_abandoned_time' );
			// Next, we update the name attribute to access this element's ID in the context of the display options array.
			// We also access the show_header element of the options collection in the call to the checked() helper function.
			printf(
				'<input type="text" id="ac_cart_abandoned_time" name="ac_cart_abandoned_time" value="%s" />',
				isset( $cart_abandoned_time ) ? esc_attr( $cart_abandoned_time ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox.
			$html = '<label for="ac_cart_abandoned_time"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for Abandoned cart time field for Guest user.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 4.0
		 */
		public static function wcap_cart_abandoned_time_guest_callback( $args ) {
			// First, we read the option
			$cart_abandoned_time_guest = get_option( 'ac_cart_abandoned_time_guest' );
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			printf(
				'<input type="text" id="ac_cart_abandoned_time_guest" name="ac_cart_abandoned_time_guest" value="%s" />',
				isset( $cart_abandoned_time_guest ) ? esc_attr( $cart_abandoned_time_guest ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="ac_cart_abandoned_time_guest"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for deleting abandoned order after X days field.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 2.3.8
		 */
		public static function wcap_delete_abandoned_orders_days_callback( $args ) {
			// First, we read the option
			$delete_abandoned_order_days = get_option( 'ac_delete_abandoned_order_days', '365' );
			if ( '' === $delete_abandoned_order_days ) {
				$delete_abandoned_order_days = '365';
			}
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			printf(
				'<input type="text" id="ac_delete_abandoned_order_days" name="ac_delete_abandoned_order_days" value="%s" />',
				isset( $delete_abandoned_order_days ) ? esc_attr( $delete_abandoned_order_days ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="ac_delete_abandoned_order_days"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for considering new abandoned cart of user after x number of days
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 8.5
		 */
		public static function wcap_cart_abandoned_after_x_days_order_placed_callback( $args ) {

			$ac_cart_abandoned_after_x_days_order_placed = get_option( 'ac_cart_abandoned_after_x_days_order_placed' );

			if ( '' == $ac_cart_abandoned_after_x_days_order_placed ){
				$ac_cart_abandoned_after_x_days_order_placed = 0;
			}

			printf(
				'<input type="number" id="ac_cart_abandoned_after_x_days_order_placed" name="ac_cart_abandoned_after_x_days_order_placed" value="%s" />',
				isset( $ac_cart_abandoned_after_x_days_order_placed ) ? esc_attr( $ac_cart_abandoned_after_x_days_order_placed ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="ac_cart_abandoned_after_x_days_order_placed"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for email admin on cart recovery field.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 2.3.8
		 */
		public static function wcap_email_admin_on_recovery_callback( $args ) {
			// First, we read the option
			$email_admin_on_recovery = get_option( 'ac_email_admin_on_recovery' );
			// This condition added to avoid the notie displyed while Check box is unchecked.
			if ( isset( $email_admin_on_recovery ) && $email_admin_on_recovery == '' ) {
				$email_admin_on_recovery = 'off';
			}
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			$html = '';
			printf(
				'<input type="checkbox" id="ac_email_admin_on_recovery" name="ac_email_admin_on_recovery" value="on"
                ' . checked( 'on', $email_admin_on_recovery, false ) . ' />'
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html .= '<label for="ac_email_admin_on_recovery"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for disable guest cart field.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 2.3.8
		 */
		public static function wcap_disable_guest_cart_email_callback( $args ) {
			// First, we read the option
			$disable_guest_cart_email = get_option( 'ac_disable_guest_cart_email' );
			// This condition added to avoid the notie displyed while Check box is unchecked.
			if ( isset( $disable_guest_cart_email ) && $disable_guest_cart_email == '' ) {
				$disable_guest_cart_email = 'off';
			}
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function.
			printf(
				'<input type="checkbox" id="ac_disable_guest_cart_email" name="ac_disable_guest_cart_email" value="on"
                ' . checked( 'on', $disable_guest_cart_email, false ) . ' />'
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox.
			echo '<label for="ac_disable_guest_cart_email"> ', esc_attr( $args[0] ), '</label> <br> <div id ="wcap_atc_disable_msg" class="wcap_atc_disable_msg"></div>';
		}

		/**
		 * Callback for disable logged-in cart email field.
		 *
		 * @param array $args Argument given while adding the field.
		 *
		 * @since 2.3.8
		 */
		public static function wcap_disable_logged_in_cart_email_callback( $args ) {
			// First, we read the option.
			$disable_logged_in_cart_email = get_option( 'ac_disable_logged_in_cart_email' );
			// This condition added to avoid the notice displyed while Check box is unchecked.
			if ( isset( $disable_logged_in_cart_email ) && $disable_logged_in_cart_email == '' ) {
				$disable_logged_in_cart_email = 'off';
			}
			// Next, we update the name attribute to access this element's ID in the context of the display options array.
			// We also access the show_header element of the options collection in the call to the checked() helper function.
			printf(
				'<input type="checkbox" id="ac_disable_logged_in_cart_email" name="ac_disable_logged_in_cart_email" value="on"
                ' . checked( 'on', $disable_logged_in_cart_email, false ) . ' />'
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox.
			echo '<label for="ac_disable_logged_in_cart_email"> ', esc_attr( $args[0] ), '</label>';
		}

		/**
		 * Callback for enabling capturing through form fields.
		 *
		 * @param array $args Argument given while adding the field.
		 *
		 * @since 8.3.0
		 */
		public static function wcap_capture_email_from_forms( $args ) {
			// First, we read the option.
			$capture_email_forms = get_option( 'ac_capture_email_from_forms' );
			// This condition added to avoid the notice displyed while Check box is unchecked.
			if ( isset( $capture_email_forms ) && '' === $capture_email_forms ) {
				$capture_email_forms = 'off';
			}
			// Next, we update the name attribute to access this element's ID in the context of the display options array.
			// We also access the show_header element of the options collection in the call to the checked() helper function.
			printf(
				'<input type="checkbox" id="ac_capture_email_from_forms" name="ac_capture_email_from_forms" value="on"
                ' . checked( 'on', $capture_email_forms, false ) . ' />'
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox.
			echo '<label for="ac_capture_email_from_forms"> ', esc_attr( $args[0] ), '</label>';
		}

		/**
		 * Callback for form fields classes.
		 *
		 * @param array $args Argument given while adding the field.
		 *
		 * @since 8.3.0
		 */
		public static function wcap_email_forms_classes( $args ) {
			// First, we read the option.
			$email_forms_classes = get_option( 'ac_email_forms_classes' );

			$readonly_param = ( get_option( 'ac_capture_email_from_forms' ) === '' ) ? 'readonly="readonly"' : '';
			// Next, we update the name attribute to access this element's ID in the context of the display options array.
			// We also access the show_header element of the options collection in the call to the checked() helper function.
			printf(
				'<textarea rows="4" cols="80" id="ac_email_forms_classes" name="ac_email_forms_classes" %s>%s</textarea>',
				esc_attr( $readonly_param ),
				isset( $email_forms_classes ) ? esc_attr( $email_forms_classes ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox.
			echo '<br><label for="ac_email_forms_classes"> ', esc_attr( $args[0] ), '</label>';
		}

		/**
		 * Call back for capturing the email from URL.
		 *
		 * @param array $args Argument given while adding the field.
		 *
		 * @since 7.6
		 */
		public static function wcap_capture_email_address_from_url( $args ) {
			// First, we read the option
			$ac_capture_email_address_from_url = get_option( 'ac_capture_email_address_from_url' );
			// Next, we update the name attribute to access this element's ID in the context of the display options array.
			// We also access the show_header element of the options collection in the call to the checked() helper function.
			printf(
				'<input type="text" id="ac_capture_email_address_from_url" name="ac_capture_email_address_from_url" value="%s" />',
				isset( $ac_capture_email_address_from_url ) ? esc_attr( $ac_capture_email_address_from_url ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox.
			$html = '<label for="ac_capture_email_address_from_url_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Call back function for guest user cart capture message.
		 *
		 * @param array $args Argument for adding field details.
		 *
		 * @since 7.8
		 */
		public static function wcap_guest_cart_capture_msg_callback( $args ) {
			$guest_msg = get_option( 'wcap_guest_cart_capture_msg' );

			$html = "<textarea rows='4' cols='80' id='wcap_guest_cart_capture_msg' name='wcap_guest_cart_capture_msg'>$guest_msg</textarea>";

			$html .= '<label for="wcap_guest_cart_capture_msg"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Call back function for registered user cart capture message.
		 *
		 * @param array $args Argument for adding field details.
		 *
		 * @since 7.8
		 */
		public static function wcap_logged_cart_capture_msg_callback( $args ) {
			$logged_msg = get_option( 'wcap_logged_cart_capture_msg' );

			$html = "<input type='text' class='regular-text' id='wcap_logged_cart_capture_msg' name='wcap_logged_cart_capture_msg' value='$logged_msg' />";

			$html .= '<label for="wcap_logged_cart_capture_msg"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Text to allow the user the choice to opt out of cart tracking
		 *
		 * @param array $args Argument for adding field details.
		 * @since 8.3
		 */
		public static function wcap_gdpr_allow_opt_out_callback( $args ) {

			$wcap_gdpr_allow_opt_out = get_option( 'wcap_gdpr_allow_opt_out' );

			$html = "<input type='text' class='regular-text' id='wcap_gdpr_allow_opt_out' name='wcap_gdpr_allow_opt_out' value='" . htmlspecialchars( $wcap_gdpr_allow_opt_out, ENT_QUOTES ) . "' />";

			$html .= '<label for="wcap_gdpr_allow_opt_out"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Message to display when the user chooses to opt out of cart tracking.
		 *
		 * @param array $args Argument for adding field details.
		 * @since 8.3
		 */
		public static function wcap_gdpr_opt_out_msg_callback( $args ) {

			$wcap_gdpr_opt_out_message = get_option( 'wcap_gdpr_opt_out_message' );

			$html = "<input type='text' id='wcap_gdpr_opt_out_message' name='wcap_gdpr_opt_out_message' value='" . htmlspecialchars( $wcap_gdpr_opt_out_message, ENT_QUOTES ) . "' style='width:60%;'/>";

			$html .= '<label for="wcap_gdpr_opt_out_message"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * GDPR section.
		 *
		 * @since 8.3
		 */
		public static function wcap_gdpr_callback() {}

		/**
		 * Option to enable GDPR consent for emails.
		 *
		 * @param array $args Argument for adding field details.
		 * @since 8.18.0
		 */
		public static function wcap_enable_gdpr_consent_callback( $args ) {
			$enable_gdpr = get_option( 'wcap_enable_gdpr_consent', '' );
			if ( '' === $enable_gdpr ) {
				$wcap_guest_cart_capture_msg  = get_option( 'wcap_guest_cart_capture_msg', '' );
				$wcap_logged_cart_capture_msg = get_option( 'wcap_logged_cart_capture_msg', '' );
				if ( ''!== $wcap_guest_cart_capture_msg || ''!== $wcap_logged_cart_capture_msg ) {
					$enable_gdpr ='on';
				}
			}
			
			?>
			<input type="checkbox" id="wcap_enable_gdpr_consent" name="wcap_enable_gdpr_consent" value="on" <?php echo checked( 'on', $enable_gdpr, false ); ?> />
			<label for="wcap_enable_gdpr_consent"> <?php echo esc_attr( $args[0] ); ?></label>
			<?php
		}

		/**
		 * Option to enable SMS consent.
		 *
		 * @param array $args Argument for adding field details.
		 * @since 8.18.0
		 */
		public static function wcap_enable_sms_consent_callback( $args ) {
			$enable_sms_consent = get_option( 'wcap_enable_sms_consent', '' );
			?>
			<input type="checkbox" id="wcap_enable_sms_consent" name="wcap_enable_sms_consent" value="on" <?php echo checked( 'on', $enable_sms_consent, false ); ?> />
			<label for="wcap_enable_sms_consent"> <?php echo esc_attr( $args[0] ); ?></label>
			<?php
		}

		/**
		 * Option for SMS consent message.
		 *
		 * @param array $args Argument for adding field details.
		 * @since 8.18.0
		 */
		public static function wcap_sms_consent_msg_callback( $args ) {
			$sms_consent_msg = get_option( 'wcap_sms_consent_msg', '' );

			$html = "<textarea rows='4' cols='80' id='wcap_sms_consent_msg' name='wcap_sms_consent_msg'>$sms_consent_msg</textarea>";

			$html .= '<label for="wcap_sms_consent_msg"><br> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Option for deleting the plugin data upon uninstall.
		 *
		 * @param array $args Argument for adding field details.
		 * @since 8.3
		 */
		public static function wcap_deleting_coupon_data( $args ) {
			$wcap_delete_coupon_data = get_option( 'wcap_delete_coupon_data', '' );
			if ( isset( $wcap_delete_coupon_data ) && '' === $wcap_delete_coupon_data ) {
				$wcap_delete_coupon_data = 'off';
				wp_clear_scheduled_hook( 'woocommerce_ac_delete_coupon_action' );
			} else {
				if ( ! wp_next_scheduled( 'woocommerce_ac_delete_coupon_action' ) ) {
				    wp_schedule_event( time(), 'wcap_15_days', 'woocommerce_ac_delete_coupon_action' );
				}
			}

			?>
			<input type="checkbox" id="wcap_delete_coupon_data" name="wcap_delete_coupon_data" value="on" <?php echo checked( 'on', $wcap_delete_coupon_data, false ); ?> />
			<label for="wcap_delete_coupon_data"> <?php echo esc_attr( $args[0] ); ?></label>
			<a >
			<?php
		}

		public static function wcap_deleting_coupon_data_manually( $args ) {
			?>
			<!-- <a href="#" class="button button-large wcap_delete_coupon"><?php //echo __( "Delete", "woocommerce-ac" ); ?></a> -->
			<input type="button" class="button-secondary" id="wcap_delete_coupons" value="<?php esc_html_e( 'Delete', 'woocommerce-ac' ); ?>" >
			<label> <?php echo esc_attr( $args[0] ); ?></label>
			<br>
			<span class="wcap-spinner" style="display:none;">
				<img class="ajax_img" src="<?php echo WCAP_PLUGIN_URL . '/assets/images/ajax-loader.gif';?>" />
			</span>
			<span class="wcap-coupon-response-msg"></span>
			<?php
		}

		/**
		 * Option for deleting the plugin data upon uninstall.
		 *
		 * @param array $args Argument for adding field details.
		 * @since 8.3
		 */
		public static function wcap_deleting_plugin_data( $args ) {
			$wcac_delete_plugin_data = get_option( 'wcac_delete_plugin_data' );
			if ( isset( $wcac_delete_plugin_data ) && '' === $wcac_delete_plugin_data ) {
				$wcac_delete_plugin_data = 'off';
			}
			?>
			<input type="checkbox" id="wcac_delete_plugin_data" name="wcac_delete_plugin_data" value="on" <?php echo checked( 'on', $wcac_delete_plugin_data, false ); ?> />
			<label for="wcac_delete_plugin_data"> <?php echo esc_attr( $args[0] ); ?></label>
			<?php
		}

		/**
		 * Option for adding product to cart when ATC is closed.
		 *
		 * @param array $args Argument for adding field details.
		 * @since 8.3
		 */
		public static function wcap_atc_close_icon_add_product_to_cart_callback( $args ) {
			$wcap_close_icon_add_product = get_option( 'wcap_atc_close_icon_add_product_to_cart', '' );
			if ( isset( $wcap_close_icon_add_product ) && '' === $wcap_close_icon_add_product ) {
				$wcap_close_icon_add_product = 'off';
			}
			?>
			<input type="checkbox" id="wcap_atc_close_icon_add_product_to_cart" name="wcap_atc_close_icon_add_product_to_cart" value="on" <?php echo checked( 'on', $wcap_close_icon_add_product, false ); ?> />
			<label for="wcap_atc_close_icon_add_product_to_cart"> <?php echo esc_attr( $args[0] ); ?></label>
			<?php
		}

		/**
		 * Callback for capturing guest cart which do not reach the checkout page.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 2.7
		 */
		public static function wcap_track_guest_cart_from_cart_page_callback( $args ) {
			// First, we read the option
			$disable_guest_cart_from_cart_page = get_option( 'ac_track_guest_cart_from_cart_page' );
			$disable_guest_cart_email          = get_option( 'ac_disable_guest_cart_email' );
			// This condition added to avoid the notice displyed while Check box is unchecked.
			if ( isset( $disable_guest_cart_from_cart_page ) && $disable_guest_cart_from_cart_page == '' ) {
				$disable_guest_cart_from_cart_page = 'off';
			}
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			$html     = '';
			$disabled = '';
			if ( isset( $disable_guest_cart_email ) && $disable_guest_cart_email == 'on' ) {
				$disabled                          = 'disabled';
				$disable_guest_cart_from_cart_page = 'off';
			}
			printf(
				'<input type="checkbox" id="ac_track_guest_cart_from_cart_page" name="ac_track_guest_cart_from_cart_page" value="on"
                ' . checked( 'on', $disable_guest_cart_from_cart_page, false ) . '
                ' . $disabled . ' />'
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html .= '<label for="ac_track_guest_cart_from_cart_page"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for Abandoned cart email settings.
		 *
		 * @since 4.7
		 */
		public static function wcap_email_callback() {
		}

		/**
		 * Callback for Abandoned cart Coupon settings.
		 *
		 * @since 4.7
		 */
		public static function wcap_coupon_callback() {
		}

		/**
		 * Callback for Abandoned cart email from name setting.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 4.7
		 */
		public static function wcap_from_name_callback( $args ) {
			// First, we read the option
			$wcap_from_name = get_option( 'wcap_from_name' );
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			printf(
				'<input type="text" id="wcap_from_name" name="wcap_from_name" value="%s" />',
				isset( $wcap_from_name ) ? esc_attr( $wcap_from_name ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="wcap_from_name_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for Abandoned cart email from email setting.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 4.7
		 */
		public static function wcap_from_email_callback( $args ) {
			// First, we read the option
			$wcap_from_email = get_option( 'wcap_from_email' );
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			printf(
				'<input type="text" id="wcap_from_email" name="wcap_from_email" value="%s" />',
				isset( $wcap_from_email ) ? esc_attr( $wcap_from_email ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="wcap_from_email_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for Abandoned cart email reply to email setting.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 4.7
		 */
		public static function wcap_reply_email_callback( $args ) {
			// First, we read the option
			$wcap_reply_email = get_option( 'wcap_reply_email' );
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			printf(
				'<input type="text" id="wcap_reply_email" name="wcap_reply_email" value="%s" />',
				isset( $wcap_reply_email ) ? esc_attr( $wcap_reply_email ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="wcap_reply_email_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for Product image size in Abandoned cart email.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 5.0
		 */
		public static function wcap_product_image_size_callback( $args ) {
			// First, we read the option
			$wcap_product_image_height = get_option( 'wcap_product_image_height' );
			$wcap_product_image_width  = get_option( 'wcap_product_image_width' );
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			?> <input type="text" id = "wcap_product_image_height" style= "width:50px" name="wcap_product_image_height" value="<?php echo $wcap_product_image_height; ?>" />
			 <?php echo 'x'; ?>
			 <input type="text" id = "wcap_product_image_width" style = "width:50px" name="wcap_product_image_width" value="<?php echo $wcap_product_image_width; ?>" />
			 px
			<?php
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="wcap_product_image_size"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for product name link redirect.
		 *
		 * @since 8.8.0
		 */
		public static function wcap_product_name_redirect_callback( $args ) {

			$wcap_product_name_redirect = get_option( 'wcap_product_name_redirect', 'checkout' );

			$pages = array(
				'checkout' => __( 'Checkout Page', 'woocommerce-ac' ),
				'product'  => __( 'Product Page', 'woocommerce-ac'),
				'none'     => __( 'No linkback', 'woocommerce-ac' ),
			);
			?>
			<select id='wcap_product_name_redirect' name='wcap_product_name_redirect'>
			<?php
			foreach ( $pages as $slug => $display ) {
				$selected = $slug === $wcap_product_name_redirect ? 'selected' : '';
				echo sprintf(
					'<option value="%s" %s>%s</option>',
					esc_attr( $slug ),
					esc_attr( $selected ),
					esc_attr( $display )
				);
			}
			?>
			</select>
			<?php
			$html = '<label for="wcap_product_name_redirect">' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for UTM parameters.
		 *
		 * @since 8.8.0
		 */
		public static function wcap_add_utm_to_links_callback( $args ) {

			$wcap_add_utm_to_links = get_option( 'wcap_add_utm_to_links', '' );

			?>
			<textarea id='wcap_add_utm_to_links' rows='4' cols='50' name='wcap_add_utm_to_links'><?php echo esc_html( $wcap_add_utm_to_links ); ?></textarea>
			<?php
			$html = '<label for="wcap_add_utm_to_links">' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for Cron job setting of the plugin.
		 *
		 * @since 4.0
		 */
		public static function wcap_cron_job_callback() {
		}

		/**
		 * Callback for use wp-cron for Abandoned cart email.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 4.0
		 */
		public static function wcap_use_auto_cron_callback( $args ) {
			// First, we read the option
			$enable_auto_cron = get_option( 'wcap_use_auto_cron' );

			// This condition added to avoid the notie displyed while Check box is unchecked.
			if ( isset( $enable_auto_cron ) && '' == $enable_auto_cron ) {
				$enable_auto_cron = 'off';
			}
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			$html = '';
			printf(
				'<input type="checkbox" id="wcap_use_auto_cron" name="wcap_use_auto_cron" value="on"
                ' . checked( 'on', $enable_auto_cron, false ) . ' />'
			);
			// $html = '<input type="checkbox" id="wcap_use_auto_cron" name="wcap_use_auto_cron" value="on" ' . checked( 'on', $enable_auto_cron, false ) . '/>';
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html .= '<label for="wcap_use_auto_cron_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for setting the cron  interval for Abandoned cart email.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 4.0
		 */
		public static function wcap_cron_time_duration_callback( $args ) {
			// First, we read the option
			$wcap_cron_time_duration = get_option( 'wcap_cron_time_duration' );
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			printf(
				'<input type="text" id="wcap_cron_time_duration" name="wcap_cron_time_duration" value="%s" />',
				isset( $wcap_cron_time_duration ) ? esc_attr( $wcap_cron_time_duration ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="wcap_cron_time_duration"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for License plugin option.
		 *
		 * @since 2.3.8
		 */
		public static function wcap_general_license_key_section_callback() {
		}

		/**
		 * Callback for Email reports option.
		 *
		 * @since 8.20.0
		 */
		public static function ac_email_reports_section_callback() {
		}

		/**
		 * Callback for Email reports frequency field.
		 *
		 * @param array $args Argument given while adding the field.
		 *
		 * @since 8.20.0
		 */
		public static function wcap_email_reports_frequency_callback( $args ) {
			$wcap_email_reports_frequency_field = get_option( 'wcap_email_reports_frequency' );
			?>
			<div class="wcap-form-group field-input">
				<div class="field-wrap email_reports_section">
					<div class="wrapper">
						<input type="radio" id="weekly_option" name="wcap_email_reports_frequency" class="form-control" value="weekly" <?php if ( $wcap_email_reports_frequency_field === 'weekly' ) { echo 'checked="checked"'; } // phpcs:ignore ?> >
						<label for="weekly_option"><?php echo esc_html__( 'Weekly', 'woocommerce-ac' ); ?></label>
					</div>
					<div class="wrapper">
						<input type="radio" id="monthly_option" name="wcap_email_reports_frequency" class="form-control" value="monthly" <?php if ( $wcap_email_reports_frequency_field === 'monthly' ) { echo 'checked="checked"'; } // phpcs:ignore ?> >
						<label for="monthly_option"><?php echo esc_html__( 'Monthly', 'woocommerce-ac' ); ?></label>
					</div>
					<div class="warning" id="email_reports_warn"><?php echo $args[0]; ?></div>			
				</div>
			</div>
			<?php
		}

		/**
		 * Callback for Email reports frequency field.
		 *
		 * @param array $args Argument given while adding the field.
		 *
		 * @since 8.20.0
		 */
		public static function wcap_email_reports_emails_list_callback( $args ) {
			$wcap_email_reports_emails_list = get_option( 'wcap_email_reports_emails_list' );
			printf(
				'<textarea rows="5" cols="50" id="wcap_email_reports_emails_list" name="wcap_email_reports_emails_list">%s</textarea>',
				isset( $wcap_email_reports_emails_list ) ? esc_attr( $wcap_email_reports_emails_list ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the field.
			$html = '<label for="wcap_email_reports_emails_list"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for License key field.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 2.3.8
		 */
		public static function wcap_edd_sample_license_key_ac_woo_callback( $args ) {
			$edd_sample_license_key_ac_woo_field = get_option( 'edd_sample_license_key_ac_woo' );
			printf(
				'<input type="text" id="edd_sample_license_key_ac_woo" name="edd_sample_license_key_ac_woo" class="regular-text" value="%s" />',
				isset( $edd_sample_license_key_ac_woo_field ) ? esc_attr( $edd_sample_license_key_ac_woo_field ) : ''
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="edd_sample_license_key_ac_woo"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for Activate License key button.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 2.3.8
		 */
		public static function wcap_activate_license_key_ac_woo_callback() {
			$license = get_option( 'edd_sample_license_key_ac_woo' );
			$status  = get_option( 'edd_sample_license_status_ac_woo' );
			?>
				<form method="post" action="options.php">
				<?php
				if ( false !== $license ) {
					?>
					<?php
					if ( $status !== false && $status == 'valid' ) {
						?>
						<span style="color:green;"><?php _e( 'active' ); ?></span>
						<?php wp_nonce_field( 'edd_sample_nonce', 'edd_sample_nonce' ); ?>
						<input type="submit" class="button-secondary" name="edd_ac_license_deactivate" value="<?php _e( 'Deactivate License' ); ?>"/>
						<?php
					} else {
						?>

								<?php
								wp_nonce_field( 'edd_sample_nonce', 'edd_sample_nonce' );
								?>

								<input type="submit" class="button-secondary" name="edd_ac_license_activate" value="<?php _e( 'Activate License' ); ?>"/>
								<?php
					}
					?>
					<?php
				}
				?>
				</form>
			<?php
		}

		/**
		 * Callback for custom restriction of the abandoned carts.
		 *
		 * @since 4.1
		 */
		public static function wcap_custom_restrict_callback() {
		}

		/**
		 * Callback for restrict IP address.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 4.1
		 */
		public static function wcap_restrict_ip_address_callback( $args ) {
			// First, we read the option
			$wcap_restrict_ip_address = get_option( 'wcap_restrict_ip_address' );
			$value                    = isset( $wcap_restrict_ip_address ) ? esc_attr( $wcap_restrict_ip_address ) : '';
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			printf(
				'<textarea rows="4" cols="50" id="wcap_restrict_ip_address" name="wcap_restrict_ip_address" placeholder="' . __( 'Add an IP address' ) . '" />' . $value . '</textarea>'
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="wcap_restrict_ip_address_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for restrict email address.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 4.1
		 */
		public static function wcap_restrict_email_address_callback( $args ) {
			// First, we read the option
			$wcap_restrict_email_address = get_option( 'wcap_restrict_email_address' );
			$email_value                 = isset( $wcap_restrict_email_address ) ? esc_attr( $wcap_restrict_email_address ) : '';
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			printf(
				'<textarea rows="4" cols="50" id="wcap_restrict_email_address" name="wcap_restrict_email_address" placeholder="' . __( 'Add an email address' ) . '" />' . $email_value . '</textarea>'
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="wcap_restrict_email_address_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for restrict domain names.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 4.1
		 */
		public static function wcap_restrict_domain_address_callback( $args ) {
			// First, we read the option
			$wcap_restrict_domain_address = get_option( 'wcap_restrict_domain_address' );
			$domain_value                 = isset( $wcap_restrict_domain_address ) ? esc_attr( $wcap_restrict_domain_address ) : '';
			// Next, we update the name attribute to access this element's ID in the context of the display options array
			// We also access the show_header element of the options collection in the call to the checked() helper function
			printf(
				'<textarea rows="4" cols="50" id="wcap_restrict_domain_address" name="wcap_restrict_domain_address" placeholder="' . __( 'Add an email domain name (Ex. hotmail.com)' ) . '" />' . $domain_value . '</textarea>'
			);
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox
			$html = '<label for="wcap_restrict_domain_address_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for restrict countries.
		 *
		 * @param array $args Argument given while adding the field.
		 *
		 * @since 8.20.0
		 */
		public static function wcap_restrict_countries_callback( $args ) {
			// First, we read the option.
			$wcap_restrict_countries = get_option( 'wcap_restrict_countries' );
			if ( ! $wcap_restrict_countries ) {
				$wcap_restrict_countries = array();
			}
			$wc_countries_object = new WC_Countries();
			$all_countries_list  = $wc_countries_object->get_countries();
			// Next, we update the name attribute to access this element's ID in the context of the display options array.
			// We also access the show_header element of the options collection in the call to the checked() helper function.
			?>
			<select class="select2" id="wcap_restrict_countries" name='wcap_restrict_countries[]' multiple="" data-placeholder="<?php echo esc_attr_e( 'Select countries', 'woocommerce-ac' ); ?>">
				<?php
				if ( ! is_array( $wcap_restrict_countries ) ) {
					$wcap_restrict_countries = (array) $wcap_restrict_countries;
				}
				foreach ( $all_countries_list as $code => $name ) {					
					$selected = in_array( $code, $wcap_restrict_countries, true ) ? 'selected' : '';
					echo sprintf(
						'<option value="%s" %s>%s</option>',
						esc_attr( $code ),
						esc_attr( $selected ),
						esc_attr( $name )
					);
				}
				?>
			</select>
			<?php
			// Here, we'll take the first argument of the array and add it to a label next to the checkbox.
			$html = '<label for="wcap_restrict_countries_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Unsubscribe settings section.
		 *
		 * @since 8.11.0
		 */
		public static function wcap_unsubscribe_options_callback() {
		}

		/**
		 * Callback for unsubscribe landing page.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 8.11.0
		 */
		public static function wcap_unsubscribe_landing_page_callback( $args ) {
			$unsubscribe_landing_page = get_option( 'wcap_unsubscribe_landing_page', 'default_page' );

			$unsubscribe_options = array(
				'default_page'   => __( 'Default Unsubscribe Page', 'woocommerce-ac' ),
				'custom_text'    => __( 'Custom Text', 'woocommerce-ac' ),
				'custom_wp_page' => __( 'Custom WordPress page', 'woocommerce-ac' )
			);

			printf( '<select id="wcap_unsubscribe_landing_page" name="wcap_unsubscribe_landing_page">' );
			foreach ( $unsubscribe_options as $u_key => $u_value ) {
				$selected = $unsubscribe_landing_page === $u_key ? 'selected' : '';
				printf(
					'<option value="%s" %s>%s</option>',
					esc_attr( $u_key ),
					esc_attr( $selected ),
					esc_html( $u_value )
				);
			}
			print( '</select>' );

			$html = '<label for="wcap_unsubscribe_landing_page_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for unsubscribe custom text.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 8.11.0
		 */
		public static function wcap_unsubscribe_custom_content_callback( $args ) {
			$initial_data = get_option( 'wcap_unsubscribe_custom_content', 'You have successfully unsubscribed and will not receive any further reminders for abandoned carts.' );

			wp_editor(
				__( $initial_data, 'woocommerce-ac' ),
				'wcap_unsubscribe_custom_content',
				array(
				'media_buttons' => true,
				'textarea_rows' => 15,
				'tabindex' => 4,
				'tinymce' => array(
					'theme_advanced_buttons1' => 'bold,italic,underline,|,bullist,numlist,blockquote,|,link,unlink,|,spellchecker,fullscreen,|,formatselect,styleselect',
				),
				)
			);

			$html = '<label for="wcap_unsubscribe_custom_content_label"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for unsubscribe custom WP page.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 8.11.0
		 */
		public static function wcap_unsubscribe_custom_wp_page_callback( $args ) {
			$custom_pages = get_option('wcap_unsubscribe_custom_wp_page');
			?>
			<select style="width:30%" class="wcap_unsubscribe_custom_wp_page wc-product-search" name="wcap_unsubscribe_custom_wp_page" data-placeholder='<?php esc_attr__( 'Search for a Page&hellip;', 'woocommerce-ac' )?>' data-action='wcap_json_find_pages'>

			<?php 
				if( $custom_pages > 0 ) {
					$post_title = get_the_title( $custom_pages );
					printf( "<option value='%s' selected>%s</option>\n", $custom_pages, $post_title );
				}
			?> 
			</select> 
			<?php
			$html = '<label for="wcap_unsubscribe_custom_wp_page">' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for SMS Settings callback.
		 *
		 * @since 7.9
		 */
		public static function wcap_sms_settings_section_callback() {
			_e( 'Configure your Twilio account settings below. Please note that due to some restrictions from Twilio, customers <i>may sometimes</i> receive delayed messages', 'woocommerce-ac' );
		}

		/**
		 * Callback for enable SMS reminders.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 7.9
		 */
		public static function wcap_enable_sms_reminders_callback( $args ) {
			$wcap_enable_sms = get_option( 'wcap_enable_sms_reminders' );

			if ( isset( $wcap_enable_sms ) && $wcap_enable_sms == '' ) {
				$wcap_enable_sms = 'off';
			}

			$html = '<input type="checkbox" id="wcap_enable_sms_reminders" name="wcap_enable_sms_reminders" value="on" ' . checked( 'on', $wcap_enable_sms, false ) . '/>';

			$html .= '<label for="wcap_enable_sms_reminders"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for From Phone Number.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 7.9
		 */
		public static function wcap_sms_from_phone_callback( $args ) {
			$wcap_from_phone = get_option( 'wcap_sms_from_phone' );

			$html = "<input type='text' id='wcap_sms_from_phone' name='wcap_sms_from_phone' value='$wcap_from_phone'  />";

			$html .= '<label for="wcap_from_phone"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for Account SID.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 7.9
		 */
		public static function wcap_sms_account_sid_callback( $args ) {
			$wcap_sms_account_sid = get_option( 'wcap_sms_account_sid' );

			$html = "<input type='text' style='width:60%;' id='wcap_sms_account_sid' name='wcap_sms_account_sid' value='$wcap_sms_account_sid'  />";

			$html .= '<label for="wcap_sms_account_sid"> ' . $args[0] . '</label>';
			echo $html;
		}

		/**
		 * Callback for Auth Token.
		 *
		 * @param array $args Argument given while adding the field
		 *
		 * @since 7.9
		 */
		public static function wcap_sms_auth_token_callback( $args ) {
			$wcap_sms_auth_token = get_option( 'wcap_sms_auth_token' );

			$html = "<input type='text' style='width:60%;' id='wcap_sms_auth_token' name='wcap_sms_auth_token' value='$wcap_sms_auth_token'  />";

			$html .= '<label for="wcap_sms_auth_token"> ' . $args[0] . '</label>';
			echo $html;
		}
	}
}
